package main

import (
	"encoding/json"
	"fmt"
	"log"
	"sync"
	"time"

	"github.com/474420502/focus/tree/heap"
	"github.com/gin-gonic/gin"
)

var tagCounter *tagCounterDict // make(map[string]*tagcounter)

type tagCounterDict struct {
	Lock            sync.Mutex
	Data            map[string]*tagcounter
	CountMethodDict map[string]func(args ...string) (*tagcounter, error)
}

type taginfo struct {
	Name  string
	Value int64
}

type tagcounter struct {
	Name      string
	LastTime  time.Time
	CountWord map[string]*taginfo
	PQueue    []interface{}
}

func init() {
	tagCounter = &tagCounterDict{}
	tagCounter.Data = map[string]*tagcounter{}
	tagCounter.CountMethodDict = make(map[string]func(args ...string) (*tagcounter, error))

	tagCounter.CountMethodDict["count"] = countTagWithCountMethod
	tagCounter.CountMethodDict["followers"] = countTagWithFollersMethod
	tagCounter.CountMethodDict["gratuity"] = countTagWithGratuityMethod

}

func AllTag(cxt *gin.Context) {
	countTagInfo(cxt, func(cxt *gin.Context, cw *tagcounter) {
		cxt.JSON(200, cw.CountWord)
	})
}

func CountTag(cxt *gin.Context) {
	countTagInfo(cxt, func(cxt *gin.Context, cw *tagcounter) {
		cxt.JSON(200, cw.PQueue)
	})
}

func countTagWithCountMethod(args ...string) (*tagcounter, error) {

	sqlcount := fmt.Sprintf(SqlTagWithCount, args[0])
	rows, err := StoreStreamer.Query(sqlcount)
	if err != nil {
		return nil, err
	}

	cw := &tagcounter{}
	cw.CountWord = make(map[string]*taginfo)
	cw.LastTime = time.Now()

	for rows.Next() {
		var stag string
		err := rows.Scan(&stag)
		if err != nil {
			log.Println(err)
		}
		var tag []string
		json.Unmarshal([]byte(stag), &tag)
		for _, t := range tag {
			if _, ok := cw.CountWord[t]; ok {
				cw.CountWord[t].Value++
			} else {
				cw.CountWord[t] = &taginfo{Name: t, Value: 1}
			}
		}
	}

	return cw, nil
}

func countTagWithGratuityMethod(args ...string) (*tagcounter, error) {

	sqlcount := fmt.Sprintf(SqlTagWithAttr, args[0])
	rows, err := StoreStreamer.Query(sqlcount)
	if err != nil {
		return nil, err
	}

	cw := &tagcounter{}
	cw.CountWord = make(map[string]*taginfo)
	cw.LastTime = time.Now()

	for rows.Next() {
		var followers, views, gratuity int64
		var tagsbuf []byte
		var tags []string
		rows.Scan(&tagsbuf, &followers, &views, &gratuity)
		json.Unmarshal(tagsbuf, &tags)
		if len(tags) > 0 {
			for _, t := range tags {
				if _, ok := cw.CountWord[t]; ok {
					cw.CountWord[t].Value += gratuity
				} else {
					cw.CountWord[t] = &taginfo{Name: t, Value: gratuity}
				}
			}
		}
	}

	return cw, nil
}

func countTagWithFollersMethod(args ...string) (*tagcounter, error) {

	sqlcount := fmt.Sprintf(SqlTagWithAttr, args[0])
	rows, err := StoreStreamer.Query(sqlcount)
	if err != nil {
		return nil, err
	}

	cw := &tagcounter{}
	cw.CountWord = make(map[string]*taginfo)
	cw.LastTime = time.Now()

	for rows.Next() {
		var followers, views, gratuity int64
		var tagsbuf []byte
		var tags []string
		rows.Scan(&tagsbuf, &followers, &views, &gratuity)
		json.Unmarshal(tagsbuf, &tags)
		if len(tags) > 0 {
			for _, t := range tags {
				if _, ok := cw.CountWord[t]; ok {
					cw.CountWord[t].Value += followers
				} else {
					cw.CountWord[t] = &taginfo{Name: t, Value: followers}
				}
			}
		}
	}

	return cw, nil
}

func countTagInfo(cxt *gin.Context, ret func(cxt *gin.Context, cw *tagcounter)) {
	platform := cxt.Query("platform")
	key := cxt.Query("countkey")

	var cw *tagcounter
	// var err error

	tagCounter.Lock.Lock()
	defer tagCounter.Lock.Unlock()

	if icw, ok := tagCounter.Data[platform+key]; ok {
		cw = icw
		if time.Now().Sub(cw.LastTime).Minutes() <= 10 {
			ret(cxt, cw)
			return
		}
	}

	// sql := fmt.Sprintf(SqlTag, platform)
	if countMehtod, ok := tagCounter.CountMethodDict[key]; ok {
		icw, err := countMehtod(platform)
		if err != nil {
			cxt.Error(err)
			return
		}
		cw = icw
	}

	cw.Name = platform
	heap := heap.New(func(a, b interface{}) int {
		if a.(*taginfo).Value >= b.(*taginfo).Value {
			return 1
		}
		return -1

	})

	var other = &taginfo{Name: "Other...", Value: 0}
	for _, v := range cw.CountWord {
		heap.Put(v)
	}

	if heap.Size() == 0 {
		cxt.JSON(200, nil)
		return
	}

	for i := 0; i <= 100; i++ {
		if v, ok := heap.Pop(); ok {
			cw.PQueue = append(cw.PQueue, v)
		}
	}

	other.Value = cw.PQueue[len(cw.PQueue)-1].(*taginfo).Value - 1
	if other.Value == 0 {
		other.Value = 1
	}

	cw.PQueue = append(cw.PQueue, other)
	ret(cxt, cw)
	cw.LastTime = time.Now()
	tagCounter.Data[platform+key] = cw
}
